import svmc

from svmc import C_SVC, C_SVCq, R2, R2q, NU_SVR
from svmc import LINEAR, POLY, RBF, SIGMOID

class svm_parameter:
	
	# default values
	default_parameters = {
	'svm_type' : C_SVC,
	'kernel_type' : RBF,
	'degree' : 3,
	'gamma' : 0,		# 1/k
	'coef0' : 0,
	'nu' : 0.5,
	'cache_size' : 40,
	'C' : 1,
	'eps' : 1e-3,
	'p' : 0.1,
	'shrinking' : 1,
	'cal_partial' : 0,
	'nr_weight' : 0,
	#'param.weight_label' : 'NULL',
	#'param.weight' : 'NULL',
	}

	def __init__(self,**kw):
		self.__dict__['param'] = svmc.new_svm_parameter()
		for attr,val in self.default_parameters.items():
			setattr(self,attr,val)
		self.__dict__['weight_label_list'] = []
		self.__dict__['weight_list'] = []
		self.__dict__['weight_label'] = svmc.int_array(0)
		self.__dict__['weight'] = svmc.double_array(0)

		for attr,val in kw.items():
			setattr(self, attr, val)

	def __getattr__(self,attr):
		get_func = getattr(svmc,'svm_parameter_%s_get' % (attr))
		return get_func(self.param)

	def __setattr__(self,attr,val):
		if attr == 'weight_label':
			svmc.int_destroy(self.weight_label)
			self.__dict__['weight_label'] = weight_label_array = svmc.int_array(len(val))
			self.__dict__['weight_label_list'] = val
			for i in range(len(val)):
				svmc.int_set(weight_label_array, i, val[i])
			val = weight_label_array
		elif attr == 'weight':
			svmc.double_destroy(self.weight)
			self.__dict__['weight'] = weight_array = svmc.double_array(len(val))
			self.__dict__['weight_list'] = val
			for i in range(len(val)):
				svmc.double_set(weight_array, i, val[i])
			self.nr_weight = len(val)
			val = weight_array
		set_func = getattr(svmc,'svm_parameter_%s_set' % (attr))
		set_func(self.param,val)

	def __repr__(self):
		ret = '<svm_parameter:'
		for name in dir(svmc):
			if name[:len('svm_parameter_')] == 'svm_parameter_' and name[-len('_set'):] == '_set':
				attr = name[len('svm_parameter_'):-len('_set')]
				if attr == 'weight_label':
					ret = ret + ' weight_label = %s,' % (self.weight_label_list)
				elif attr == 'weight':
					ret = ret + ' weight = %s,' % (self.weight_list)
				else:
					ret = ret+' %s = %s,' % (attr,getattr(self,attr))
		return ret+'>'

	def __del__(self):
		svmc.delete_svm_parameter(self.param)
		svmc.int_destroy(self.weight_label)
		svmc.double_destroy(self.weight)

def _convert_to_svm_node_array(x):
	""" convert a sequence or mapping to an svm_node array """
	data = svmc.svm_node_array(len(x)+1)
	svmc.svm_node_array_set(data,len(x),-1,0)
	import operator
	if operator.isMappingType(x):
		keys = x.keys()
		keys.sort()
		j = 0
		for k in keys:
			svmc.svm_node_array_set(data,j,k,x[k])
			j = j + 1
	elif operator.isSequenceType(x):
		for j in range(len(x)):
			svmc.svm_node_array_set(data,j,j+1,x[j])
	else:
		raise TypeError,"data must be a mapping or a sequence"
	
	return data

class svm_problem:
	def __init__(self,y,x):
		assert len(y) == len(x)
		self.prob = prob = svmc.new_svm_problem()
		self.size = size = len(y)

		self.y_array = y_array = svmc.double_array(size)
		for i in range(size):
			svmc.double_set(y_array,i,y[i])

		self.x_matrix = x_matrix = svmc.svm_node_matrix(size)
		self.data = []
		self.maxlen = 0;
		for i in range(size):
			data = _convert_to_svm_node_array(x[i])
			self.data.append(data);
			svmc.svm_node_matrix_set(x_matrix,i,data)
			self.maxlen = max(self.maxlen,len(x[i]))

		svmc.svm_problem_l_set(prob,size)
		svmc.svm_problem_y_set(prob,y_array)
		svmc.svm_problem_x_set(prob,x_matrix)

	def __repr__(self):
		return "<svm_problem: size = %s>" % (self.size)

	def __del__(self):
		svmc.delete_svm_problem(self.prob)
		svmc.double_destroy(self.y_array)
		for i in range(self.size):
			svmc.svm_node_array_destroy(self.data[i])
		svmc.svm_node_matrix_destroy(self.x_matrix)

class svm_model:
	def __init__(self,arg1,arg2=None):
		if arg2 == None:
			# create model from file
			filename = arg1
			self.model = svmc.svm_load_model(filename)
		else:
			# create model from problem and parameter
			prob,param = arg1,arg2
			assert(len(param.weight_list) == len(param.weight_label_list))
			if param.gamma == 0:
				param.gamma = 1.0/prob.maxlen
			self.model = svmc.svm_train(prob.prob,param.param)

	def predict(self,x):
		data = _convert_to_svm_node_array(x)
		ret = svmc.svm_predict(self.model,data)
		svmc.svm_node_array_destroy(data)
		return ret

	def save(self,filename):
		svmc.svm_save_model(filename,self.model)

	def __getattr__(self,attr):
		if attr == 'partial':
			self.partial = svmc.svm_calculate_partial(self.model)
			return self.partial
		if attr == 'sv_coef':
			ret = []
			for i in range(self.l):
				ret.append([])
				for j in range(self.nr_class - 1):
					ret[i].append(svmc.matrix_double_get(svmc.svm_model_sv_coef_get(self.model), j, i))
			self.sv_coef = ret
			return ret
		if attr == 'SV':
			ret = []
			for i in range(self.l):
				ret.append({})
				j = 0
				while svmc.matrix_svm_node_index_get(svmc.svm_model_SV_get(self.model), i, j) != -1:
					ret[i][svmc.matrix_svm_node_index_get(svmc.svm_model_SV_get(self.model), i, j)] = svmc.matrix_svm_node_value_get(svmc.svm_model_SV_get(self.model), i, j)
					j += 1;
			self.SV = ret
			return ret
		if attr == 'nSV':
			ret = []
			for i in range(self.nr_class):
				ret.append(svmc.int_get(svmc.svm_model_nSV_get(self.model), i))
			self.nSV = ret
			return ret
		if attr == 'nBSV':
			ret = []
			for i in range(self.nr_class):
				ret.append(svmc.int_get(svmc.svm_model_nBSV_get(self.model), i))
			self.nBSV = ret
			return ret
		get_func = getattr(svmc,'svm_model_%s_get' % (attr))
		return get_func(self.model)

	def __del__(self):
		svmc.svm_destroy_model(self.model)
